`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: Worcester Polytechnic Institute
// Engineer: Kazim Shaikh
// Integration by: Dario Martinovic, Michael Padberg
// 
// Create Date: 01/21/2018
// Module Name: IR_top
//
// Description: Top module for controlling IR sensor integration with VGA
//              color display and sound buzzer.
// 
//////////////////////////////////////////////////////////////////////////////////


module IR_top(

    
    input [1:0] IR_data,               //input from IR sensor (HIGH if there is a detection) 
    //data loaded from ps AXI transmission     
    input [7:0] AXI_slv_reg10,      //high if object is in danger warning zone
    input [7:0] AXI_slv_reg11,      //high if object is in critical action zone
    output sound,                   //output to a speaker/buzzer
    output reg [1:0] color_signal,  //color selection for color_logic_inst
  
    //VGA controller signals 
    input vga_reset_N,           // active low reset coming from PS 
    output [11:0] clr_rgb_top, // takes values that define the color display of the VGA
    output HS_top,             // Hsync output to the VGA
    output VS_top,             // Vsync output to the VGA

    //input clock signals
    input clk_25M,         // 25 MHz from MMCM
    input clk_100M          //100MHz - FPGA/AXI Clock

);
    
    wire blank_wire;           // a blank signal to the VGA
  
    wire [10:0] hcount_wire;   // hcount output to the VGA
    wire [10:0] vcount_wire;   // vcount output to the VGA
    
    reg [21:0] tone;           // tone for buzzer
     
    reg IR_detection;          // Combined data from two IRs


    //detection for sound output determined from IR & Lidar 
    always @ (posedge clk_100M) begin
        IR_detection <= IR_data[0] | IR_data[1];  //Two IR sensor signals ORed for single detection
    end
        
    always @ (AXI_slv_reg10, AXI_slv_reg11, IR_detection) begin
        if((AXI_slv_reg11 > 0) && IR_detection) begin   //if object in critical zone, IR's detect person
            color_signal = 2'b10;   // show red
            tone = 22'd30000; //play sound
        end
        else if (AXI_slv_reg10 > 0) begin       //if object is in danger warning zone
            color_signal = 2'b01;   //show yellow
            tone = 22'd0000;
        end
        else begin
            color_signal = 2'b00;   //show green
            tone = 22'd0000;
        end
    end
    
    // Frequency Divider
    tone_generator tone_generator_inst( .clk(clk_100M),
        .counter(tone),
        .sound(sound)
    );
    
    // This module is creating outputs that will be manipulated by the color logic module to display 
    // graphics on the monitor.  VGA controller is Digilent IP.
    vga_controller_640_60 in3 (.pixel_clk(clk_25M),
                               .rst(vga_reset_N), 
                               .HS(HS_top), 
                               .VS(VS_top), 
                               .hcount(hcount_wire), 
                               .vcount(vcount_wire), 
                               .blank(blank_wire)); 
    
    //RGB color logic for VGA port                           
    color_logic color_logic_inst (.blank(blank_wire), 
                                .vcount(vcount_wire),
                                .color_signal(color_signal),
                                .hcount(hcount_wire), 
                                .clr_rgb(clr_rgb_top));
                          
    
endmodule






