`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: Worcester Polytechnic Institute
// Engineers: Dario Martinovic, Michael Padberg
// 
// Create Date: 11/19/2017 01:14:27 PM
// Module Name: lidar_controller
//
// Description:  Programmable logic based state machine for controlling
//                       data transmission to from LeddarTech LeddarVu8.
//////////////////////////////////////////////////////////////////////////////////


module lidar_controller(

    //Physical board I/O
    input reset,    //BTND
    input fpga_CLK, //Zedboard clock
    input MISO,     //Master in slave out data line
    input start_lidar_btn, //BTNU
    output CS_N,    //Chip select lane
    output clk_1M_L,   //Clock output to lidar device
    output MOSI,        //Master out slave in
    
    //Data output to PS over Axi
    output reg [31:0] scale_received_data,  //scale data to PS
    output reg [34:0] segment_7, //MSB byte = Distance MSB byte, LSB byte = Segment #
                      segment_6,
                      segment_5,
                      segment_4,
                      segment_3,
                      segment_2,
                      segment_1,
                      segment_0 
    );
    
    //state machine
    reg [3:0] current_state, 
              next_state;
     
    //Parameters for state labels
    parameter [3:0] 
              //read distance scale states
              state_start = 4'b0000,
              state_load_scale = 4'b0001,
              state_scale_command = 4'b0010,
              state_delay_scale = 4'b0011,
              state_reading_scale = 4'b0100,
              state_store_scale = 4'b0101,
              //read data states
              state_wait = 4'b0110,
              state_load = 4'b0111,
              state_read_command = 4'b1000,
              state_reading = 4'b1001,
              state_delay = 4'b1010,
              state_store_reading = 4'b1011;
             // state_stop = 4'b1011;
    
    //Lidar control opcodes for commands
    parameter [7:0] LIDAR_READ_OPCODE = 8'h0B,
                    LIDAR_WRITE_OPCODE = 8'h02,
                    LIDAR_CE_OPCODE = 8'hC7;
    //LeddarVu8 memory bank base addresses          
    parameter [23:0] BANK_0_START_ADDRESS = 24'h000000,  //Configuration Data
               BANK_5_START_ADDRESS = 24'h400000,  //Device Information and constants
               BANK_13_START_ADDRESS = 24'h500000, //Detection list
               BANK_19_START_ADDRESS = 24'hFFFB00; //Transaction configuration
    
    //Offsets applied to memory bank addresses for data access           
    parameter [23:0] FPGA_Version_Offset = 24'h120; //Register offset of FPGA Version in Bank 5
    parameter [23:0] Detection_List_Array_Offset = 24'hc; //Register offset of Detection List Array in Bank 13
    parameter [23:0] Distance_Scale_Offset = 24'h162; // Register offset of Distance Scale in Bank 5
               
    //used to generate 1kHz clock
    reg [16:0] count_100M_1K; //count from 0 to 99999
    parameter [16:0] MAXIMUM_COUNT_1K = 100000;
    wire clk_enable_1K;
    
    
     //used to generate 2Hz clock
    reg [25:0] count_100M_2;
    parameter [25:0] MAXIMUM_COUNT_2 = 26'd50000000;
    wire clk_enable_2;
    
    
     //used to generate 1MHz clock
    parameter [6:0] MAXIMUM_COUNT_1M = 7'd100;
    reg [6:0] count100M_1M;
    wire clk_1M_posedge;
    reg clk_1M_enabled = 1'b1;
    
    //send buffer used to send data out on MOSI pin
    reg [47:0] send_buffer;
    reg [7:0] count_shifts_W = 8'h00;
    
    
    //receive buffer used to load in lidar response
    reg [767:0] receive_buffer;
    reg [31:0] scale_receive_buffer; 
    reg [9:0] count_shifts_R = 10'h0;
    reg [6:0] scale_count_shifts_R = 7'b0;
     
    
    //sequential logic for FSM
    always @ (posedge fpga_CLK, negedge reset) begin
       if(reset == 1'b0)begin
           current_state <= state_start;
       end
       else begin
           current_state <= next_state;
       end
    end
    
    //combinational next state logic
    always @ (current_state,clk_enable_1K,count_shifts_W,count_shifts_R,scale_count_shifts_R,clk_enable_2,start_lidar_btn)begin
        case(current_state)
            state_start: begin
                if(start_lidar_btn)     //wait for start button signal
                    next_state = state_load_scale;
                else
                    next_state = state_start;
            end
            state_load_scale: begin
                next_state = state_scale_command;
            end
            state_scale_command: begin
                if(count_shifts_W == 8'h30)
                    next_state = state_delay_scale;
                else
                    next_state = state_scale_command;   
            end
            state_delay_scale: begin
                if(clk_enable_1K)
                    next_state = state_reading_scale;
                else
                    next_state = state_delay_scale;
            end
            state_reading_scale: begin
                if(scale_count_shifts_R == 7'h20)   //hex 20 = 32 decimal.  Count in 32 bits. 
                    next_state = state_store_scale;
                else
                    next_state = state_reading_scale;            
            end
            state_store_scale: begin
                    next_state = state_wait;
            end
            state_wait: begin   //wait for enable signal from counter
                if(clk_enable_2) //acquire_distance_data
                    next_state = state_load;
                else
                    next_state = state_wait;
            end
            
            state_load:begin
                next_state = state_read_command;
            end
            
            state_read_command:begin
                if(count_shifts_W == 8'h30)
                    next_state = state_delay;
                else
                    next_state = state_read_command;            
            end
            
            state_delay:begin
                if(clk_enable_1K)  //1ms has elapsed
                    next_state = state_reading;
                else
                    next_state = state_delay;
            end
            
            state_reading:begin
                if(count_shifts_R == 12'h300) //hex 300 = 768 decimal.  Count in 768 bits.
                    next_state = state_store_reading;
                else
                    next_state = state_reading;
            end
            
            state_store_reading:begin
                next_state = state_wait;
            end

            default:
                next_state = state_wait; 
        endcase
    end
    
    //individual state logic 
    always @ (posedge fpga_CLK) begin
        case(current_state)
            state_start: begin  //first state, wait for start button signal
                count_shifts_W <= 8'b0;
                count_shifts_R <= 10'b0;
                scale_count_shifts_R <= 7'b0;
                clk_1M_enabled <= 1'b1;
                send_buffer <= 48'b0;
                scale_receive_buffer <= 32'b0;
                scale_received_data <= scale_received_data;
                receive_buffer <= 768'b0;
                
                segment_7 <= segment_7;
                segment_6 <= segment_6;
                segment_5 <= segment_5;
                segment_4 <= segment_4;
                segment_3 <= segment_3;
                segment_2 <= segment_2;
                segment_1 <= segment_1;
                segment_0 <= segment_0;
            end
            state_load_scale: begin     //load scale read command into send buffer
                clk_1M_enabled <= 1'b1; // enable clk_1M   
                send_buffer <= {LIDAR_READ_OPCODE,BANK_5_START_ADDRESS | Distance_Scale_Offset,16'h0004};
                //send_buffer <= {LIDAR_READ_OPCODE,BANK_5_START_ADDRESS | FPGA_Version_Offset,16'h0020};
                count_shifts_W <= 8'b0;
                count_shifts_R <= 10'b0;
                scale_count_shifts_R <= 7'b0;
                
                scale_receive_buffer <= 32'b0; 
                scale_received_data <= scale_received_data;
                receive_buffer <= 768'b0; 
                
                segment_7 <= segment_7;
                segment_6 <= segment_6;
                segment_5 <= segment_5;
                segment_4 <= segment_4;
                segment_3 <= segment_3;
                segment_2 <= segment_2;
                segment_1 <= segment_1;
                segment_0 <= segment_0;              
            end
            state_scale_command: begin  //send scale read command
                if(clk_1M_posedge) begin
                    count_shifts_W <= count_shifts_W + 8'b1;
                    send_buffer <= {send_buffer[46:0], 1'b0};
                end
                else begin
                    count_shifts_W <= count_shifts_W;
                    send_buffer <= send_buffer;
                
                end
                
                clk_1M_enabled <= 1'b1;
                count_shifts_R <= 10'b0;
                scale_count_shifts_R <= 7'b0;
                scale_receive_buffer <= 32'b0;
                scale_received_data <= scale_received_data;
                receive_buffer <= 768'b0;
                
                segment_7 <= segment_7;
                segment_6 <= segment_6;
                segment_5 <= segment_5;
                segment_4 <= segment_4;
                segment_3 <= segment_3;
                segment_2 <= segment_2;
                segment_1 <= segment_1;
                segment_0 <= segment_0;
            end
            state_delay_scale: begin    //delay between command and data reading
                clk_1M_enabled <= 1'b0; //disable clock output
                count_shifts_W <= 8'b0;
                count_shifts_R <= 10'b0;
                scale_count_shifts_R <= 7'b0;
                send_buffer <= 48'b0;
                scale_receive_buffer <= 32'b0;
                scale_received_data <= scale_received_data;
                receive_buffer <= 768'b0;
                
                segment_7 <= segment_7;
                segment_6 <= segment_6;
                segment_5 <= segment_5;
                segment_4 <= segment_4;
                segment_3 <= segment_3;
                segment_2 <= segment_2;
                segment_1 <= segment_1;
                segment_0 <= segment_0;
            end
            state_reading_scale: begin  //read in lidar response
                if(clk_1M_posedge) begin
                    scale_count_shifts_R <= scale_count_shifts_R + 8'b1;
                    scale_receive_buffer <= {scale_receive_buffer[30:0], MISO}; //receive and shift left
                end
                else begin
                    scale_count_shifts_R <= scale_count_shifts_R;   
                    scale_receive_buffer <= scale_receive_buffer; 
                end
                
                clk_1M_enabled <= 1'b1; 
                count_shifts_W <= 8'b0;
                count_shifts_R <= 10'b0;
                send_buffer <= 48'b0;
                scale_received_data <= scale_received_data;
                receive_buffer <= 768'b0;
                
                segment_7 <= segment_7;
                segment_6 <= segment_6;
                segment_5 <= segment_5;
                segment_4 <= segment_4;
                segment_3 <= segment_3;
                segment_2 <= segment_2;
                segment_1 <= segment_1;
                segment_0 <= segment_0;
            end
            state_store_scale: begin    //store the scale value
                clk_1M_enabled <= 1'b1; 
                scale_receive_buffer <= scale_receive_buffer;
                scale_received_data <= scale_receive_buffer[31:0];
                count_shifts_W <= 8'b0;
                count_shifts_R <= 10'b0;
                send_buffer <= 48'b0;
                scale_count_shifts_R <= 7'b0;
                receive_buffer <= 768'b0;
                segment_7 <= segment_7;
                segment_6 <= segment_6;
                segment_5 <= segment_5;
                segment_4 <= segment_4;
                segment_3 <= segment_3;
                segment_2 <= segment_2;
                segment_1 <= segment_1;
                segment_0 <= segment_0;
            end
            
            state_load: begin   //load the distance reading command into send buffer
//                send_buffer <= {LIDAR_READ_OPCODE,BANK_5_START_ADDRESS | FPGA_Version_Offset,16'h0020};
                send_buffer <= {LIDAR_READ_OPCODE,BANK_13_START_ADDRESS | Detection_List_Array_Offset,16'h0060};
                clk_1M_enabled <= 1'b1; // enable clk_1M
                count_shifts_W <= 8'b0;
                count_shifts_R <= 10'b0;
                scale_count_shifts_R <= 7'b0;
                scale_receive_buffer <= 32'b0;
                scale_received_data <= scale_received_data; 
                receive_buffer <= 768'b0;
                
                segment_7 <= segment_7;
                segment_6 <= segment_6;
                segment_5 <= segment_5;
                segment_4 <= segment_4;
                segment_3 <= segment_3;
                segment_2 <= segment_2;
                segment_1 <= segment_1;
                segment_0 <= segment_0;
            end
            state_read_command: begin   //send distance read command 
                if(clk_1M_posedge) begin    //bit shifted on each positive 1M clock edge
                    count_shifts_W <= count_shifts_W + 8'b1;
                    send_buffer <= {send_buffer[46:0], 1'b0};
                end
                else begin
                    count_shifts_W <= count_shifts_W;
                    send_buffer <= send_buffer;
                end
                
                clk_1M_enabled <= 1'b1;
                count_shifts_R <= 10'b0;
                scale_count_shifts_R <= 7'b0;
                scale_receive_buffer <= 32'b0;
                scale_received_data <= scale_received_data; 
                receive_buffer <= 768'b0;
                
                segment_7 <= segment_7;
                segment_6 <= segment_6;
                segment_5 <= segment_5;
                segment_4 <= segment_4;
                segment_3 <= segment_3;
                segment_2 <= segment_2;
                segment_1 <= segment_1;
                segment_0 <= segment_0;
            end
            state_delay:begin   //delay between command and data reading
                clk_1M_enabled <= 1'b0; //disable clk_1M
                count_shifts_R <= 10'b0;
                scale_count_shifts_R <= 7'b0;
                scale_receive_buffer <= 32'b0;
                scale_received_data <= scale_received_data;
                count_shifts_W <= 8'b0;
                send_buffer <= 48'b0;
                receive_buffer <= 768'b0;
                
                segment_7 <= segment_7;
                segment_6 <= segment_6;
                segment_5 <= segment_5;
                segment_4 <= segment_4;
                segment_3 <= segment_3;
                segment_2 <= segment_2;
                segment_1 <= segment_1;
                segment_0 <= segment_0;
            end
            state_reading:begin //read in segment distance data from lidar
                clk_1M_enabled <= 1'b1; 
                scale_count_shifts_R <= 7'b0;
                scale_receive_buffer <= 32'b0;
                scale_received_data <= scale_received_data;
                count_shifts_W <= 8'b0;
                send_buffer <= 48'b0; 
                
                segment_7 <= segment_7;
                segment_6 <= segment_6;
                segment_5 <= segment_5;
                segment_4 <= segment_4;
                segment_3 <= segment_3;
                segment_2 <= segment_2;
                segment_1 <= segment_1;
                segment_0 <= segment_0;
          
                if(clk_1M_posedge) begin
                    count_shifts_R <= count_shifts_R + 8'b1;
                    receive_buffer <= {receive_buffer[766:0], MISO}; //receive and shift left
                end
                
                else begin
                    count_shifts_R <= count_shifts_R;
                    receive_buffer <= receive_buffer;
                end
            end
            state_store_reading: begin  //store the distance readings
                clk_1M_enabled <= 1'b1; 
                scale_count_shifts_R <= 7'b0;
                scale_receive_buffer <= 32'b0;
                scale_received_data <= scale_received_data;
                count_shifts_W <= 8'b0;
                count_shifts_R <= 10'b0;
                send_buffer <= 48'b0;
                receive_buffer <= receive_buffer;
                //received data is parsed for each segment.
                segment_7 <= {receive_buffer[767:736],receive_buffer[698:696]};
                segment_6 <= {receive_buffer[671:640],receive_buffer[602:600]};
                segment_5 <= {receive_buffer[575:544],receive_buffer[506:504]};
                segment_4 <= {receive_buffer[479:448],receive_buffer[410:408]};
                segment_3 <= {receive_buffer[383:352],receive_buffer[314:312]};
                segment_2 <= {receive_buffer[287:256],receive_buffer[218:216]};
                segment_1 <= {receive_buffer[191:160],receive_buffer[122:120]};
                segment_0 <= {receive_buffer[95:64],receive_buffer[26:24]};
            end
            state_wait:begin    //wait for enable signal from counter
                clk_1M_enabled <= 1'b1;
                count_shifts_W <= 8'b0;
                count_shifts_R <= 10'b0;
                receive_buffer <= 768'b0;
                send_buffer <= 48'b0;
                scale_receive_buffer <= 32'b0;
                scale_received_data <= scale_received_data;
                scale_count_shifts_R <= 7'b0;
                 
                segment_7 <= segment_7;
                segment_6 <= segment_6;
                segment_5 <= segment_5;
                segment_4 <= segment_4;
                segment_3 <= segment_3;
                segment_2 <= segment_2;
                segment_1 <= segment_1;
                segment_0 <= segment_0;
            end

        
        endcase
    end
    
    assign MOSI = send_buffer[47];  //serial output is sent out from MSB of send buffer.
    
    //chip select is pulled low during command & data transmission.
    assign CS_N = !(current_state == state_read_command || current_state == state_delay || current_state == state_reading ||
                     current_state == state_scale_command || current_state == state_delay_scale || current_state == state_reading_scale );
    
    //creates a 1KHz clock        
    always @ (posedge fpga_CLK) begin
        if (count_100M_1K == MAXIMUM_COUNT_1K - 1)
            count_100M_1K <= 0; 
        else 
            count_100M_1K <= count_100M_1K + 1'b1; 
    end
    
    assign clk_enable_1K = (count_100M_1K == MAXIMUM_COUNT_1K - 1); //clocked at 1KHz
    
   //creates a 2Hz clock        
    always @ (posedge fpga_CLK) begin
        if (count_100M_2 == MAXIMUM_COUNT_2 - 1)
            count_100M_2 <= 0; 
        else 
            count_100M_2 <= count_100M_2 + 1'b1; 
    end

    assign clk_enable_2 = (count_100M_2 == MAXIMUM_COUNT_2 - 1); //clocked at 2Hz
    
    
    //creates a 1MHz clock
    always @ (posedge fpga_CLK) begin
        if (count100M_1M == MAXIMUM_COUNT_1M - 4'b1)
            count100M_1M <= 0; 
        else
            count100M_1M <= count100M_1M + 4'b1;
    end
                        
    assign clk_1M_L = (count100M_1M < 5) && clk_1M_enabled;  //50% duty cycle clock at 1MHz rate          
    assign clk_1M_posedge = (count100M_1M == 0);   
    
endmodule
