`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: Worcester Polytechnic Institute
// Engineers: Dario Martinovic, Michael Padberg
// 
// Create Date: 11/19/2017 12:59:46 PM
// Module Name: pl_lidar_top_module
// 
// Description: Top module to control passing of data between lidar controller,
//              Infrared Sensor/VGA warning top module, and LED warning module.
// 
//////////////////////////////////////////////////////////////////////////////////


module pl_lidar_top_module(

    //Physical board I/O
    input fpga_CLK, //Zedboard clock
    input reset, //BTND
    output CS_N_L,  //chip select line
    output MOSI_L,  //Master out slave in data line
    output clk_1M_L,   //Clocking signal to lidar device
    input MISO_L,   //Master in slave out data line
    input start_lidar_btn, //BTNU
    
    //Outputs for PS connection over AXI.
    output [31:0] distance_scale_out,
    output [31:0] segment7, //distance reading for segments 7-0
    output [31:0] segment6,
    output [31:0] segment5,
    output [31:0] segment4,
    output [31:0] segment3,
    output [31:0] segment2,
    output [31:0] segment1,
    output [31:0] segment0,
    output [31:0] seg_nums  //segment nums corresponding to distance readings. 
                            //seg_nums[2:0] corresponds to segment0 distance reading, etc.
    
    );
      
    wire [31:0] scale_received_data;    //Scale data from lidar control module
    
    //Segment distances - raw data from lidar control module
    wire [34:0] segment_7,      //MSB byte = Distance MSB byte, LSB byte = Segment #
                segment_6,
                segment_5,
                segment_4,
                segment_3,
                segment_2,
                segment_1,
                segment_0; 
    
    //distance scale value passed to PS over AXI for distance conversion to meters           
    assign distance_scale_out = {scale_received_data[7:0],scale_received_data[15:8],scale_received_data[23:16],scale_received_data[31:24]};
    
    //individual segment distance readings passed to PS over AXI. Raw sample data.
    assign segment7 = {segment_7[10:3],segment_7[18:11],segment_7[26:19],segment_7[34:27]};
    assign segment6 = {segment_6[10:3],segment_6[18:11],segment_6[26:19],segment_6[34:27]};
    assign segment5 = {segment_5[10:3],segment_5[18:11],segment_5[26:19],segment_5[34:27]};
    assign segment4 = {segment_4[10:3],segment_4[18:11],segment_4[26:19],segment_4[34:27]};
    assign segment3 = {segment_3[10:3],segment_3[18:11],segment_3[26:19],segment_3[34:27]};
    assign segment2 = {segment_2[10:3],segment_2[18:11],segment_2[26:19],segment_2[34:27]};
    assign segment1 = {segment_1[10:3],segment_1[18:11],segment_1[26:19],segment_1[34:27]};
    assign segment0 = {segment_0[10:3],segment_0[18:11],segment_0[26:19],segment_0[34:27]};
    assign seg_nums = {8'b0, segment_7[2:0], segment_6[2:0],segment_5[2:0],segment_4[2:0],segment_3[2:0],segment_2[2:0],segment_1[2:0],segment_0[2:0]};
    
    //Lidar control logic instance
    lidar_controller lidar_controller_inst (
    .reset(reset),
    .fpga_CLK(fpga_CLK),
    .MISO(MISO_L),
    .CS_N(CS_N_L),
    .clk_1M_L(clk_1M_L),
    .MOSI(MOSI_L),
    .start_lidar_btn(start_lidar_btn),
    .scale_received_data(scale_received_data),
    .segment_7(segment_7),
    .segment_6(segment_6),
    .segment_5(segment_5),
    .segment_4(segment_4),
    .segment_3(segment_3),
    .segment_2(segment_2),
    .segment_1(segment_1),
    .segment_0(segment_0) 
    ); 
    
endmodule
