/******************************************************************************
*
* Copyright (C) 2009 - 2014 Xilinx, Inc.  All rights reserved.
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* Use of the Software is limited solely to applications:
* (a) running on a Xilinx device, or
* (b) that interact with a Xilinx device through a bus or interconnect.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
* XILINX  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
* WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
* OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* Except as contained in this notice, the name of the Xilinx shall not be used
* in advertising or otherwise to promote the sale, use or other dealings in
* this Software without prior written authorization from Xilinx.
*
******************************************************************************/

/*
 *   ps7_uart is configured to 115200
 *   Authors: Michael Padberg & Dario Martinovic
 */

#include "platform.h"
#include "xbasic_types.h"
#include "xparameters.h"
#include "xil_printf.h"
#include <stdio.h>

#define NUMSEGMENTS	8
#define DISTANCETHRESHOLD 1.0
#define DANGERWARNTHRESHOLD	3.9624 //3.9624 meters
#define CRITACTIONTHRESHOLD	2.4384 //2.4384 meters


//function prototypes
void testSegmentThreshold(float dist, int segNum);
void checkSegmentThresholds(float dist, int segNum);
void delay_time();


//Global Variables

//pointer to base address of PS-PL memory location shared over AXI.
Xuint32 *baseaddr_p = (Xuint32 *)XPAR_LIDAR_AXI_0_S00_AXI_BASEADDR;

//bitmasks
const Xuint32 SEG7_NUM = 0x00E00000;
const Xuint32 SEG6_NUM = 0x001C0000;
const Xuint32 SEG5_NUM = 0x00038000;
const Xuint32 SEG4_NUM = 0x00007000;
const Xuint32 SEG3_NUM = 0x00000E00;
const Xuint32 SEG2_NUM = 0x000001C0;
const Xuint32 SEG1_NUM = 0x00000038;
const Xuint32 SEG0_NUM = 0x00000007;

//raw readings from shared PS-PL memory
Xuint32 distance_scale = 0;
Xuint32 distance_seg7 = 0;
Xuint32 distance_seg6 = 0;
Xuint32 distance_seg5 = 0;
Xuint32 distance_seg4 = 0;
Xuint32 distance_seg3 = 0;
Xuint32 distance_seg2 = 0;
Xuint32 distance_seg1 = 0;
Xuint32 distance_seg0 = 0;
Xuint32 seg_nums = 0;


//segment distances converted to meters.
float segMeters[NUMSEGMENTS];


//segment numbers parsed from seg_nums reading.
int distanceSegNums[NUMSEGMENTS];

int main()
{

	init_platform();

	//begin operating loop
	while(1)
	{

		xil_printf("Getting reading....\n\r");

		//Short delay to give the Lidar some time to acquire distance data before next computation
		delay_time(2500000);

		xil_printf("Reading ready\n\r");

		//get raw distance readings from shared PS-PL memory
		distance_scale = *(baseaddr_p+9);
		distance_seg7 = *(baseaddr_p+7);
		distance_seg6 = *(baseaddr_p+6);
		distance_seg5 = *(baseaddr_p+5);
		distance_seg4 = *(baseaddr_p+4);
		distance_seg3 = *(baseaddr_p+3);
		distance_seg2 = *(baseaddr_p+2);
		distance_seg1 = *(baseaddr_p+1);
		distance_seg0 = *(baseaddr_p+0);
		seg_nums = *(baseaddr_p+8);

		//print scale value for conversion
		xil_printf("Distance Scale : 0x%08x \n\r", distance_scale);

		//print seg_nums output from memory
		xil_printf("Segment Nums Read : 0x%08x \n\r", seg_nums);


		//Calculate Distance Reading in Meters
		segMeters[7] = ((float)distance_seg7) / ((float) distance_scale);
		segMeters[6] = ((float)distance_seg6) / ((float) distance_scale);
		segMeters[5] = ((float)distance_seg5) / ((float) distance_scale);
		segMeters[4] = ((float)distance_seg4) / ((float) distance_scale);
		segMeters[3] = ((float)distance_seg3) / ((float) distance_scale);
		segMeters[2] = ((float)distance_seg2) / ((float) distance_scale);
		segMeters[1] = ((float)distance_seg1) / ((float) distance_scale);
		segMeters[0] = ((float)distance_seg0) / ((float) distance_scale);


		//Decode seg nums for printing using bitmask
		distanceSegNums[7] = (int) (seg_nums & SEG7_NUM) >> 21;
		distanceSegNums[6] = (int) (seg_nums & SEG6_NUM) >> 18;
		distanceSegNums[5] = (int) (seg_nums & SEG5_NUM) >> 15;
		distanceSegNums[4] = (int) (seg_nums & SEG4_NUM) >> 12;
		distanceSegNums[3] = (int) (seg_nums & SEG3_NUM) >> 9;
		distanceSegNums[2] = (int) (seg_nums & SEG2_NUM) >> 6;
		distanceSegNums[1] = (int) (seg_nums & SEG1_NUM) >> 3;
		distanceSegNums[0] = (int) (seg_nums & SEG0_NUM);


		//check all segment readings for distances within object detection threshold
		for(int i=0; i<NUMSEGMENTS; i++)
		{
			checkSegmentThresholds(segMeters[i], distanceSegNums[i]);
		}

		// Distance Reading
		printf("Seg7 Distance Reading : %f \t Segment: %i \n\r", segMeters[7], distanceSegNums[7]);
		printf("Seg6 Distance Reading : %f \t Segment: %i \n\r", segMeters[6], distanceSegNums[6]);
		printf("Seg5 Distance Reading : %f \t Segment: %i \n\r", segMeters[5], distanceSegNums[5]);
		printf("Seg4 Distance Reading : %f \t Segment: %i \n\r", segMeters[4], distanceSegNums[4]);
		printf("Seg3 Distance Reading : %f \t Segment: %i \n\r", segMeters[3], distanceSegNums[3]);
		printf("Seg2 Distance Reading : %f \t Segment: %i \n\r", segMeters[2], distanceSegNums[2]);
		printf("Seg1 Distance Reading : %f \t Segment: %i \n\r", segMeters[1], distanceSegNums[1]);
		printf("Seg0 Distance Reading : %f \t Segment: %i \n\r", segMeters[0], distanceSegNums[0]);


		xil_printf("End of test\n\n\r");

	} //end while(1) operation loop

	return 0;

} //end main



/*
 * Tests segment distances to see if they are below the 1m testing distance threshold.
 * Segments lower than threshold are flagged in DDR memory to pass to PL.
 *
 * @param dist		The distance in one segment from the LeddarVu, converted to meters.
 * @param segNum	The segment number associated with the distance reading.
 */
void testSegmentThreshold(float dist, int segNum)
{
	Xuint32 flagToWrite = 0x00000001;

	if( dist < DISTANCETHRESHOLD )
	{
		*(baseaddr_p+10) |= (flagToWrite << segNum);	//left shift by segment number to write flag.
	}													//flag written high if distance in segment is below threshold
	else
	{
		*(baseaddr_p+10) &= ~(flagToWrite << segNum);	//flag written low if distance in segment is above threshold
	}
}



/*
 * Checks segment distances to see if they are below the Critical Action and
 * Danger Warning zone thresholds.
 * Segments lower than threshold are flagged in DDR memory to pass to PL.
 * Critical Action - slv_reg11
 * Danger Warning - slv_reg10
 *
 * @param dist		The distance in one segment from the LeddarVu, converted to meters.
 * @param segNum	The segment number associated with the distance reading.
 */
void checkSegmentThresholds(float dist, int segNum)
{
	Xuint32 flagToWrite = 0x00000001;

	if( dist < CRITACTIONTHRESHOLD )
	{
		*(baseaddr_p+11) |= (flagToWrite << segNum);	//Critical action written to slv_reg11
		*(baseaddr_p+10) |= (flagToWrite << segNum);	//Danger warning written to slv_reg10
														//left shift by segment number to write flag.
	}													//flag written high if distance in segment is below threshold
	else
	{
		*(baseaddr_p+11) &= ~(flagToWrite << segNum);	//Critical action flag written low

		if( dist < DANGERWARNTHRESHOLD )
		{													//Danger warning written to slv_reg10
			*(baseaddr_p+10) |= (flagToWrite << segNum);	//left shift by segment number to write flag.
		}													//flag written high if distance in segment is below threshold
		else
		{
			*(baseaddr_p+10) &= ~(flagToWrite << segNum);	//Danger warning flag written low
															//flag written low if distance in segment is above threshold
		}
	}

}



/*
 * Software polling delay.
 *
 * @param t Number of iterations to poll for delay.
 */
void delay_time(long t)
{
	long i;
	for(i= t; i>0; i--)
	{
	}
}




